import requests

from .base import MessageHandler
from ..util import parse_int

_XKCD_API_CURRENT_URL = 'http://xkcd.com/info.0.json'
_XKCD_URL_TEMPLATE = 'http://xkcd.com/%s/'

class XkcdHandler(MessageHandler):

  TRIGGERS = ['xkcd']
  HELP = 'link to the given xkcd comic; default current'

  def handle_message(self, event, triggers, query):
    number = parse_int(query)
    if not number:
      response = requests.get(_XKCD_API_CURRENT_URL)
      response.raise_for_status()
      number = response.json()['num']
    return _XKCD_URL_TEMPLATE % number



import itertools

from .base import MessageHandler
from .. import settings

class ReactionHandler(MessageHandler):

  TRIGGER_ANCHOR = ''
  TRIGGER_PREFIX = ''
  TRIGGERS = sorted(
      settings.EMOJI_REACTIONS.keys() + settings.MESSAGE_REACTIONS.keys())
  HELP = 'add emoji and message reactions'

  def handle_message(self, event, triggers, query):
    for trigger in triggers:
      trigger = trigger.lower()
      for reaction in self._get_reactions(settings.EMOJI_REACTIONS, trigger):
        self.client.api_call(
            'reactions.add',
            name=reaction,
            channel=event['channel'],
            timestamp=event['ts'])
    return '\n'.join(itertools.chain.from_iterable(
        self._get_reactions(settings.MESSAGE_REACTIONS, trigger)
        for trigger in triggers))

  def _get_reactions(self, reaction_defs, trigger):
    reactions = reaction_defs.get(trigger, [])
    return [reactions] if isinstance(reactions, basestring) else reactions



import logging
import os

API_TOKEN = os.environ.get('SLACK_RTM_BOT_API_TOKEN')

LOG_FILE = None
LOG_FORMAT = '%(asctime)s: %(levelname)s: %(message)s'
LOG_LEVEL = logging.INFO

READ_INTERVAL_SECONDS = 1.0

# OwnerHandler
CHANNEL_OWNERS = {}

# ReactionHandler
EMOJI_REACTIONS = {}
MESSAGE_REACTIONS = {}



from collections import Counter
import random

from .base import MessageHandler
from ..util import parse_int

class CoinHandler(MessageHandler):

  TRIGGERS = ['coin', 'flip']
  HELP = 'flip the given number of coins; default 1'

  def handle_message(self, event, triggers, query):
    times = parse_int(query, default=1)
    flips_gen = (random.choice(('heads', 'tails')) for _ in xrange(times))
    if times > 10:
      counter = Counter(flips_gen)
      return '%s heads, %s tails' % (counter['heads'], counter['tails'])
    else:
      return ', '.join(flips_gen)



import logging
import time

from slackclient import SlackClient

from . import settings
from .handlers.base import handlers, init_handlers

def get_responses(event):
  for handler in handlers:
    try:
      yield handler.handle(event)
    except Exception:
      logging.exception('handler failed to handle event: %s', handler)

def run_bot(client):
  while True:
    logging.debug('rtm_read')
    for event in client.rtm_read():
      if 'ok' in event:
        if event['ok']:
          logging.debug('response confirmation: %s', event['reply_to'])
        else:
          logging.error('bad response confirmation: %s', event)
        continue

      logging.info('event: %s', event['type'])
      response = '\n'.join(filter(None, get_responses(event)))
      if response:
        logging.info('response: %s', response)
        client.rtm_send_message(event['channel'], response)

    time.sleep(settings.READ_INTERVAL_SECONDS)

def main():
  logging.basicConfig(filename=settings.LOG_FILE,
                      format=settings.LOG_FORMAT,
                      level=settings.LOG_LEVEL)

  client = SlackClient(settings.API_TOKEN)
  logging.info('rtm_connect')
  if not client.rtm_connect():
    logging.critical('failed to connect')
    return

  init_handlers(client)
  run_bot(client)

if __name__ == '__main__':
  main()



from .base import MessageHandler

class Rot13Handler(MessageHandler):

  TRIGGERS = ['rot13', 'rot']
  HELP = 'apply ROT13 to the given text'

  def handle_message(self, event, triggers, query):
    return query.encode('rot13')
